/**/
/*											
	Apple Computer, Inc.  2/21/91 			
	      All Rights Reserved.				

*/

#ifndef __AEOBJECTS__
#define __AEOBJECTS__

#ifndef __MEMORY__
#include <Memory.h>
#endif


#ifndef __OSUTILS__
#include <OSUtils.h>
#endif

#ifndef __EVENTS__
#include <Events.h>
#endif

#ifndef __EPPC__
#include <EPPC.h>
#endif

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif


/* Logical operators: look for them in descriptors of type typeLogicalDescriptor
  with keyword keyAELogicalOperator */
#define kAEAND						'AND '
#define kAEOR						'OR  '
#define kAENOT						'NOT '

/* Absolute ordinals: look for them in descriptors of type typeAbsoluteOrdinal.
  Possible use is as keyAEKeyData in an object specifier whose keyAEKeyForm
  field is formAbsolutePosition.
*/
#define kAEFirst					'firs'
#define kAELast						'last'
#define kAEMiddle					'midd'
#define kAEAny						'any '
#define kAEAll						'all '

/*  Relative ordinals: look for them in descriptors of type formRelativePosition.
*/
#define kAENext						'next'
#define kAEPrevious					'prev'

/********** keywords for getting fields out of Object specifier records **********/
#define keyAEDesiredClass			'want'
#define keyAEContainer				'from'
#define keyAEKeyForm				'form'
#define keyAEKeyData	 		 	'seld'


/********** keywords for getting fields out of Range specifier records **********/
#define keyAERangeStart				'star'	/* these are the only two fields in the range desc */
#define keyAERangeStop				'stop'

/********** possible values for the keyAEKeyForm field of an object specifier **********/
/* remember that this set is an open-ended one.  We make assumptions about some of them,
  but nothing in the grammar says folks can't define their own */
#define formAbsolutePosition 		'indx'		/* eg 1st, -2nd ( 2nd from end) */
#define formRelativePosition		'rele'		/* next, previous */
#define formTest					'test'		/* a logical or a comparison */
#define formRange					'rang'		/* two arbitrary objects and everything in between */
#define formPropertyID				'prop'		/* key data is a 4-char property name */
#define formName					'name'		/* key data may be of type 'TEXT' */

/************** various relevant types ****************/
/* Some of these tend to be paired with certain of the forms above.  Where this
  is the case I've included the form(s) in comments following */
#define typeObjectSpecifier 	 	'obj '		/*keyAEContainer will often be one of these*/
#define typeObjectBeingExamined 	'exmn'		/*another possible value for keyAEContainer*/
#define typeCurrentContainer		'ccnt'		/*another possible value for keyAEContainer*/
#define typeToken					'toke'		/*substituted for 'ccnt' before accessor called*/

#define typeRelativeDescriptor 		'rel '		/*formRelativePosition*/
#define typeAbsoluteOrdinal 		'abso'		/*formAbsolutePosition*/
#define typeIndexDescriptor			'inde'		/*formAbsolutePosition*/
#define typeRangeDescriptor 	 	'rang'		/*formRange*/
#define typeLogicalDescriptor	 	'logi'		/*formTest is this or typeCompDescriptor*/
#define typeCompDescriptor			'cmpd'		/*formTest is this or typeLogicalDescriptor*/

#define typeEnumeration				'enum'		/*a type of many uses*/

/************** various relevant keywords ****************/
#define keyAECompOperator			'relo'		/* relates two terms stuff like '', '<', etc. */
#define keyAELogicalTerms			'term'		/* an AEList of terms to be related by 'logc' below */
#define keyAELogicalOperator		'logc'		/* kAEAND,  kAEOR or kAENOT */
#define keyAEObject1				'obj1'		/* One of two objects in a term must be object specifier */
#define keyAEObject2				'obj2'		/* The other object may be a simple descriptor or obj. spec. */


/************ Special Handler selectors for OSL Callbacks ***************/
/* You don't need to use these unless you are not using AESetObjectCallbacks. */
#define keyDisposeTokenProc			'xtok'
#define keyAECompareProc 		 	'cmpr'
#define keyAECountProc 			 	'cont'
#define keyAEMarkTokenProc 		 	'mkid'
#define keyAEMarkProc 			 	'mark'
#define keyAEAdjustMarksProc 	 	'adjm'

/************ Error codes ***************/
/* My error codes: AEM proper uses up to -1719 */
#define errAEImpossibleRange		-1720	/* a range like 3rd to 2nd, or 1st to all. */
#define errAEWrongNumberArgs		-1721 /* logical op kAENOT used with other than 1 term */

#define errAEAccessorNotFound 		-1723	/* same as Donn's ErrNotFound */
#define errAENoSuchLogical			-1725 	/* something other than AND, OR, or NOT */
#define errAEBadTestKey				-1726	/* test is neither typeLogicalDescriptor
											nor typeCompDescriptor */
#define errAENotAnObjSpec			-1727 	/* param to AEResolve not of type 'obj ' */
#define errAENoSuchObject			-1728 	/* eg: specifier asked for the 3rd, but there are only 2.
												Basically, this indicates a run-time resolution error. */
#define errAENegativeCount			-1729	/* CountProc returned negative value */
#define errAEEmptyListContainer		-1730	/* attempt to pass empty list as container to accessor */
	/* add more here ...*/


/* possible values for flags parameter to AEResolve.  They're additive */
#define kAEIDoMinimum				0x0000 
#define kAEIDoWhose					0x0001 
#define kAEIDoMarking				0x0004 


/* you only care about the constants that follow if you're doing your own whose
  clause resolution */
#define typeWhoseDescriptor			'whos'
#define formWhose					'whos'
#define typeWhoseRange				'wrng'
#define keyAEWhoseRangeStart		'wstr'
#define keyAEWhoseRangeStop			'wstp'
#define keyAEIndex					'kidx'
#define keyAETest					'ktst'

struct ccntTokenRecord {		/* used for rewriting tokens in place of 'ccnt' descriptors		*/
		DescType tokenClass ;	/* This record is only of interest to those who, when they...	*/
		AEDesc token ;			/* ...get ranges as key data in their accessor procs, choose...	*/
	} ;							/* ...resolve them manually rather than call AEResolve again.	*/
	
typedef struct ccntTokenRecord ccntTokenRecord, *ccntTokenRecPtr, **ccntTokenRecHandle ;

typedef AEDesc *DescPtr, **DescHandle ;

/* typedefs providing type checking for procedure pointers */

typedef pascal OSErr (*accessorProcPtr) ( DescType desiredClass,
		const AEDesc *container, DescType containerClass, DescType form,
		const AEDesc *selectionData, AEDesc *value, long LongInt ) ;

typedef pascal OSErr (*compareProcPtr)( DescType oper, const AEDesc *obj1,
			const AEDesc *obj2, Boolean *result ) ;
typedef pascal OSErr (*countProcPtr)( DescType desiredType, DescType containerClass,
			const AEDesc *container, long *result ) ;
typedef pascal OSErr (*disposeTokenProcPtr)( AEDesc *unneededToken ) ;

typedef pascal OSErr (*getMarkTokenProcPtr)( const AEDesc *dContainerToken, DescType containerClass,
			AEDesc *result ) ;


typedef void *getErrDescProcPtr ;		/* a placeholder: ignore for now */

/* Note: app is supposed to dispose of dToken after marking */
typedef pascal OSErr (*markProcPtr)( const AEDesc *dToken, const AEDesc *markToken, long index ) ;
typedef pascal OSErr (*adjustMarksProcPtr)( long newStart, long newStop, const AEDesc *markToken ) ;



/* PUBLIC PROCEDURES */


pascal OSErr  AEObjectInit() ;
/* Not done by inline, but by direct linking into code.  It sets up the pack
  such that further calls can be via inline */

pascal OSErr AESetObjectCallbacks( compareProcPtr myCompareProc,
							countProcPtr myCountProc,
							disposeTokenProcPtr myDisposeTokenProc,	
							getMarkTokenProcPtr myGetMarkTokenProc,			/* called when mark (below) is true (all 3)*/
							markProcPtr myMarkProc,
							adjustMarksProcPtr myAdjustMarksProc,
							void *reserved )
= { 0x303C, 0x0E35, 0xA816 } ;




pascal OSErr  AEResolve		(	const AEDesc *objectSpecifier,
								short callbackFlags,				/* see above for possible values */
					  			AEDesc *theToken )
= { 0x303C, 0x0536, 0xA816 } ;


pascal OSErr AEInstallObjectAccessor( DescType desiredClass,
							   	DescType containerType,
							   	accessorProcPtr theAccessor,
							   	long accessorRefcon,
							   	Boolean isSysHandler )
= { 0x303C, 0x0937, 0xA816 } ;

pascal OSErr AERemoveObjectAccessor(	DescType desiredClass,
								DescType containerType,
								accessorProcPtr theAccessor,
								Boolean isSysHandler )
= { 0x303C, 0x0738, 0xA816 } ;

pascal OSErr AEGetObjectAccessor(	DescType desiredClass,
								DescType containerType,
								accessorProcPtr *accessor,
								long *accessorRefcon,
								Boolean isSysHandler )
= { 0x303C, 0x0939, 0xA816 } ;

pascal OSErr AEDisposeToken( AEDesc *theToken )
= { 0x303C, 0x023A, 0xA816 } ;

pascal OSErr AECallObjectAccessor(	DescType desiredClass,
									const AEDesc *containerToken,
									DescType containerClass,
									DescType keyForm,
									const AEDesc *keyData,
									AEDesc *token )
= { 0x303C, 0x0C3B, 0xA816 } ;




/*
Here are the interfaces your callback procs must be written to:

pascal OSErr MyCompareProc( DescType comparisonOperator, const AEDesc *theObject,
		const AEDesc *descOrObj, Boolean *result ) ;
pascal OSErr MyCountProc( DescType desiredType, DescType containerClass,
			const AEDesc *container, long *result:  ) ;
pascal OSErr MyGetMarkToken( const AEDesc *containerToken, DescType containerClass,
		AEDesc *result ) ;
pascal OSErr MyMark( const AEDesc *theToken, const AEDesc *markToken, long markCount ) ;
pascal OSErr MyAdjustMarks( long newStart, long newStop, const AEDesc *markToken ) ;
pascal OSErr MyMyDisposeToken( AEDesc *unneededToken ) ;

pascal OSErr MyObjectAccessor( DescType desiredClass, const AEDesc *containerToken,
		DescType containerClass, DescType keyForm, const AEDesc *keyData,
		AEDesc *theToken, long *theRefcon ) ;

You'll probably want to have a number of these last ones.
A proc that finds a line within a document should be installed with 'line' and 'docu'
as the desiredClass and containerClass fields in the call to AEInstallObjectHandler().
*/

#endif
